/*===========================================================================*/
/*
 * This file is part of
 * libalinga - a C++ library for Alligned Linguistic Annotation
 *
 * Copyright (C) 2007 - ...  Elaine Tsiang YueLien
 *
 * libalinga is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file Manifold.H                                                         *
 *                                                                           *
 *  ALingA::Manifold interface                                               *
 *                                                                           */
/*===========================================================================*/
#ifndef	ManifoldInterface
#define	ManifoldInterface

#ifdef __GNUG__
#pragma interface
#endif

#include <cstddef>

namespace ALingA
{
  /// @brief Space time co-ordinate template class
  ///
  /// @ingroup derivables
  template<class T>
  class CoOrd
  {
  public:
    T time;
    T space;

  public:
    CoOrd(
	  T	tm
	  ,T	sp
	  )
      : time(tm)
      , space(sp)
    {}
  };
  
  /// @brief ALingA::Position is unsigned.
  ///
  /// As opposed to Ogg::Position, which is signed.
  typedef 	size_t			Position;
  typedef	CoOrd<size_t>		Decimation;
  typedef 	CoOrd<Position>		Coordinate;


  class Manifold;
}

%template(Coordinate)	ALingA::CoOrd<size_t>;

#include <ALingA/Signal.H>
#include <ALingA/XMLwriter.H>

namespace ALingA
{
  /// @brief Manifold attributes entity
  ///
  /// See http:///www.ihear.com/dtds/ALingA/0.1/ManifoldAttrs.entity
  class Manifold
  {
  public:
    Manifold(
	     const size_t		perTime
	     ,const size_t		perSpace = 1
	     ,const Glib::ustring	timeUnit = "Sec"
	     ,const Glib::ustring	spaceUnit = ""
	     ,const Position		timeBias = 0
	     ,const Position		spaceBias = 0
	     ,const size_t		gransPerFrame = 1
	     ,const Glib::ustring	timeOrigin = ""
	     );

    Manifold(
	     const Manifold &
	     ,const ALingA::Signal &
	     );

    Manifold(
	     const Manifold &
	     );

    virtual
    ~Manifold();

    Manifold &
    operator =(const Manifold &);

    bool
    operator ==(const Manifold &) const;

    /// @brief If this manifold can be a base of the argument.
    ///
    /// Assuming identity of units and identity of time origin,
    /// the base bias must lead to inclusion of more samples
    /// and the base sampling must be of a higher resolution by integral multiples.
    /// Returns the integral multiples as Decimation.
    /// Throws exceptions otherwise.
    Decimation
    operator / (const Manifold&);

    size_t
    perTime() const;

    size_t
    perSpace() const;

    Glib::ustring
    timeUnit() const;

    Glib::ustring
    spaceUnit() const;
    
    Position
    timeBias() const;

    Position
    spaceBias() const;

    size_t
    gransPerFrame() const;

    Glib::ustring
    timeOrigin() const;

  };
}

#endif

