/*===========================================================================*/
/*
 * This file is part of
 * libalinga - a C++ library for Alligned Linguistic Annotation
 *
 * Copyright (C) 2007 - ...  Elaine Tsiang YueLien
 *
 * libalinga is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301, USA
 *
 *===========================================================================*/
/** @file Signal.H                                                           *
 *                                                                           *
 *  ALingA::Signal interface                                                 *
 *                                                                           */
/*===========================================================================*/
#ifndef	SignalInterface
#define	SignalInterface

#ifdef __GNUG__
#pragma interface
#endif

namespace ALingA
{
  class Signal;
}

#include <ALingA/Manifold.H>
#include <ALingA/Metas.H>
#include <Ogg/Exception.H>

namespace ALingA
{
  /// Non-derivable
  class Signal
  {
  private:
    Signal(const Signal &);

    Signal &
    operator=(const Signal &);

  protected:
    Signal(){};

  public:
    /// @Brief Call this to indicate to ALingA this signal is ready for writing.
    ///
    /// Throws InvalidCodecType if codecType is empty.
    /// A Signal or LingA encoder has already called ALingA::add to obtain
    /// Signal instance. It then invokes this method to indicate
    /// to the ALingA encoder it is ready to write.
    void
    beginWrite(Glib::ustring	codecType);

    /// @Brief Call this to indicate to ALingA this signal is finished for writing
    ///
    /// The ALingA encoder write loop will exit when all signals have finished.
    /// Use this method to control when ALingA may exit its write loop.
    /// This is true of the non-interleaved signals, which should be handled
    /// by the associated LingA instance.
    void
    endWrite();

    /// @Brief Call this to indicate to ALingA caller no longer needs the Signal instance
    ///
    /// The ALingA encoder read loop will persist until no Signal instance is needed.
    /// Use this method to control when ALingA may exit its read loop.
    /// This is true of the non-interleaved signals, and ignored signals.
    /// Some agent, which could be the associated LingA instance, or a controller
    /// should be responsible for calling this for every Signal instance obtained
    /// from an ALingA instance.
    void
    endRead();

    long
    track() const;

    size_t
    baseGransPerFrame() const;

    Coordinate
    bias() const;

    Decimation
    decimation() const;

    void
    href(Glib::ustring);

    const Glib::ustring &
    href() const;

    void
    meta(MetaName
	 ,MetaValue
	 );

    const MetaValue &
    meta(const MetaName);

    const Glib::ustring &
    codecType() const;

    Coordinate
    baseCoordinate(Coordinate signalCoordinate) const;

    Coordinate
    signalCoordinate(Coordinate baseCoordinate) const;
    
    Position
    basePosition(Coordinate	signalCoordinate) const;

    Position
    basePosition(Position	time
		 ,Position	space
		 ) const;

  };
}
#endif

